#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Collections.Generic;
using System.Windows.Forms;
using Gom;

#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("Enter the description of your new custom indicator here")]
	
	public class GomMP : GomDeltaIndicator
    {
		
		const int PRE_RTH=-1;
		const int POST_RTH=Int32.MaxValue;
		const int ALLBARS=Int32.MinValue;
		
		
		class VolAndDelta
		{
			
			private float _delta;
			private float _totalvolume;

			public float Delta {get {return _delta;}}
			public float TotalVolume{get {return _totalvolume;}}

            public VolAndDelta(float delta, float volume) 
			{
				_delta=delta;
				_totalvolume=volume;	
			}
			
			public void AddVolume(float delta, float volume)
			{
				_delta += delta;
				_totalvolume +=volume;
			}
			
		}		
		
	
        class VolAtPrice : Dictionary<int, VolAndDelta> 
		{
			public VolAtPrice():base() {}
			
			public VolAtPrice(int price, float delta,float volume)
			{
                this.Add(price, new VolAndDelta(delta, volume));
			}
			
			public void AddVolume(int price,  float delta,float volume)		
			{
                VolAndDelta oldVolume;
				
				if (this.TryGetValue(price,out oldVolume))
					oldVolume.AddVolume(delta,volume);
				else
                    this.Add(price, new VolAndDelta(delta, volume));
								
			}
			
			//returns a smoothed volatprice object.
			public VolAtPrice(VolAtPrice initVol,int smooth)
			{				
				float vol2add;
				float divisor=1.0f/((smooth+1)*(smooth+1));
				float curVol;
				int curPrice;
								
				foreach(KeyValuePair<int,VolAndDelta> priceiterator in initVol)
				{				
					curVol=priceiterator.Value.TotalVolume;
					curPrice=priceiterator.Key;
																	
					for (int i=1;i<=smooth;i++)
					{					
						vol2add  =curVol*(smooth+1-i)*divisor;
						this.AddVolume(curPrice+i,0,vol2add);
						this.AddVolume(curPrice-i,0,vol2add);												
					}	
					
					vol2add =curVol*(smooth+1)*divisor;
					this.AddVolume(curPrice,0,vol2add);
							
				}
			}
			
			public void Draw(GraphicalHelper helper, Color color, int x0, Dictionary<int,float> lastVolumes)
			{
				int curPrice;
				float curVol;
				float curDelta;
				float lastVol;
				
				VolAtPrice toDraw;
				
				if ((helper.colorMode!=Gom.MPColorMode.Delta) && (helper.nbSmooth>0))
					toDraw=new VolAtPrice(this,helper.nbSmooth);
				else
					toDraw=this;
				
				foreach(KeyValuePair<int,VolAndDelta> priceiterator in toDraw)
				{
					curPrice=priceiterator.Key;
					curVol=priceiterator.Value.TotalVolume;
					
					if (!lastVolumes.TryGetValue(curPrice,out lastVol))
						lastVolumes.Add(curPrice,0.0f);
					
					if (helper.colorMode==Gom.MPColorMode.Delta)
					{
						curDelta=priceiterator.Value.Delta;
						color=(curDelta>0)?helper.chartcontrol.UpColor:helper.chartcontrol.DownColor;
						
						helper.FillRectangle(curPrice,color,x0,lastVol,Math.Abs(curDelta));
						helper.FillRectangle(curPrice,helper.neutralColor,x0,lastVol+Math.Abs(curDelta),curVol-Math.Abs(curDelta));
					}
					else
						helper.FillRectangle(curPrice,color,x0,lastVol,curVol);
					
					lastVolumes[curPrice]=lastVol+curVol;	
				}	
			
			}	
		
		}
		
		//OHLC Bar
		
		class VolAtTimeBar:Dictionary<int,VolAtPrice>
		{
								
			public void AddVolume(int time,int price,float delta,float volume)
			{
	
				VolAtPrice oldVolAtPrice;
				
				if (this.TryGetValue(time,out oldVolAtPrice))
					oldVolAtPrice.AddVolume(price,delta,volume);
				else
					this.Add(time,new VolAtPrice(price,delta,volume));	
			}
		}

		class PriceVol:IComparable
		{
			private int _price;
			private float _vol;
			
			public int Price{get{return _price;}}
			public float Vol{get{return _vol;}}
			
			public PriceVol(int price, float vol)
			{
				_price=price;
				_vol=vol;
			}
			
			public PriceVol(KeyValuePair<int,float> iterator)
			{
				_price=iterator.Key;
				_vol=iterator.Value;
			}
			
			public PriceVol(PriceVol pv)
			{
				_price=pv.Price;
				_vol=pv.Vol;
			}
			
			
			public int CompareTo(object a)
			{
				return this._price.CompareTo((((PriceVol)a)._price));
			}	
		}
		
		class Profile:List<PriceVol>
			
		{
			public PriceVol POC,VWAP,VAH,VAL;
			private float totvol=0.0f;
			private GraphicalHelper helper;
			
			public Profile(GraphicalHelper param,Dictionary<int,float> lastVolumes):base()
			{
			
				helper=param;
				
				float maxvol=0;
				int maxprice=0;
				float weightedsum=0;					
									
				foreach(KeyValuePair<int,float> iterator in lastVolumes)
				{
					totvol+=iterator.Value;
					weightedsum+=iterator.Key*iterator.Value;
				
					this.Add(new PriceVol(iterator));
					
					if (iterator.Value>maxvol)
					{
						maxvol=iterator.Value;
						maxprice=iterator.Key;
				}	}


				POC=new PriceVol(maxprice,maxvol);
				
				VWAP=null;				
				if (totvol>0)
				{
					int vwapprice=Convert.ToInt32(weightedsum/totvol);
					
					if (lastVolumes.ContainsKey(vwapprice))
						VWAP=new PriceVol(vwapprice,lastVolumes[vwapprice]);
				}
				
				this.Sort();
			}

			private void ComputeVA()
			{
				if (totvol<1.0f) return;
				
				int startindex=this.BinarySearch(new PriceVol(POC.Price,0));
				int lastCommittedUpPrice=startindex;
				int lastCommittedDownPrice=startindex;		
				float CommittedVolume=POC.Vol;
				float Vol_70Pcent=0.7f*totvol;
				
				bool overflowwith2=false;
				
				float upvol,downvol;
				
				while (CommittedVolume<Vol_70Pcent)
				{	
					// 2 bars
					if ((!overflowwith2) && (lastCommittedUpPrice<this.Count-2) && (lastCommittedDownPrice>=2))
					{
						upvol=this[lastCommittedUpPrice+1].Vol+this[lastCommittedUpPrice+2].Vol;
						downvol=this[lastCommittedDownPrice-1].Vol+this[lastCommittedDownPrice-2].Vol;
						
						if (((CommittedVolume+upvol)>Vol_70Pcent) || ((CommittedVolume+downvol)>Vol_70Pcent))
							overflowwith2=true;
						else
						{
							if (upvol>downvol)
							{
								lastCommittedUpPrice+=2;
								CommittedVolume+=upvol;
							}
							else if (upvol<downvol)
							{
								lastCommittedDownPrice-=2;
								CommittedVolume+=downvol;
							}
							else
							{
								if ((CommittedVolume+upvol+downvol)>Vol_70Pcent)
									overflowwith2=true;
								else
								{
									lastCommittedUpPrice+=2;
									lastCommittedDownPrice-=2;
									CommittedVolume += upvol+downvol;
					}	}	}	}
					else
					{
						if (lastCommittedUpPrice==this.Count-1)
						{
							CommittedVolume+=this[lastCommittedDownPrice-1].Vol;
							lastCommittedDownPrice -=1;
						}
						else if (lastCommittedDownPrice==0)
						{
							CommittedVolume+=this[lastCommittedUpPrice+1].Vol;
							lastCommittedUpPrice +=1;							
						}
						else
						{
							upvol=this[lastCommittedUpPrice+1].Vol;
							downvol=this[lastCommittedDownPrice-1].Vol;

							if (upvol>downvol)
							{
								lastCommittedUpPrice+=1;
								CommittedVolume+=upvol;
							}
							else if (upvol<downvol)
							{
								lastCommittedDownPrice-=1;
								CommittedVolume+=downvol;
							}
							else		
							{
									lastCommittedUpPrice+=1;
									lastCommittedDownPrice-=1;
									CommittedVolume += upvol+downvol;
				
				}	}	}	}							
								
				VAH=new PriceVol(this[lastCommittedUpPrice]);
				VAL=new PriceVol(this[lastCommittedDownPrice]);
			}
					
			private void DrawLevel(int x0, PriceVol param, Color color)
			{
				if (param != null)
					helper.FillRectangleSpecialBars(param.Price,color, x0,(helper.allPocVolume)?POC.Vol:param.Vol);	
			}
			
			public void DrawLevels(int x0)			
			{
				if (totvol>0)
				{						
					if (helper.showVa)
					{
						ComputeVA();
						
						DrawLevel(x0,VAH,helper.colorVa);
						DrawLevel(x0,VAL,helper.colorVa);
					}
					
					if (helper.showVwap)
						DrawLevel(x0,VWAP,helper.colorVwap);
					
					if (helper.showPoc)
						DrawLevel(x0,POC,helper.colorPoc);
																		
			}	}
			
			public void DrawVN(int x0,int maxx)
			{	
				//peakdet by Eli Billauer http://www.billauer.co.il/peakdet.html
							
				float mn = float.MaxValue;
				float mx = float.MinValue;
							
				int mnpos = -1;
				int mxpos = -1;
			
				bool lookformax = true;
				
				float delta=POC.Vol*helper.vnSize/100f;
			
				float  curVol;
				
				for(int i=0;i<this.Count;i++)
				{		
					curVol =this[i].Vol;
					
					if (curVol > mx)
					{
						mx = curVol; 
						mxpos = this[i].Price;
					}
					
					if (curVol < mn)
					{
						mn = curVol;
						mnpos = this[i].Price;
					}			
			
					if (lookformax)
					{
						if (curVol< mx-delta)
						{
							helper.DrawLine(mxpos,Color.Green,x0,mx,maxx);
							mn = curVol; mnpos = this[i].Price;
							lookformax = false;
						}
					}
					else
					{
						if (curVol > mn+delta)
						{
							helper.DrawLine(mnpos,Color.Red,x0,mn,maxx);
							mx = curVol; mxpos =  this[i].Price;
							lookformax = true;
			}	}	}	}

		}
		

	
		class VolAtTimeBars: Dictionary<int,VolAtTimeBar> 
		{
			
			private GraphicalHelper helper;		
			
			public VolAtTimeBars(GraphicalHelper param):base()
			{
				helper=param;
			}
									
			static private bool isRTH (int time)
			{
				return ((time !=PRE_RTH) && (time != POST_RTH)&& (time != ALLBARS));
			}									

			private void Draw(int firstBar, int lastBar, int x0,out Dictionary<int,float> lastVolumes)
			{		
				lastVolumes=new Dictionary<int,float>();
			
				VolAtTimeBar barstodraw=new VolAtTimeBar();
				VolAtPrice curVolAtPrice=null;
						
				Dictionary<int,bool> timelist = new Dictionary<int,bool>();
				SortedDictionary<int, bool> timesorted=new SortedDictionary<int,bool>();
				
				if (!(helper.colorMode==Gom.MPColorMode.TimeColor))
				{
					timesorted.Add(ALLBARS,true);
					curVolAtPrice=new VolAtPrice();
					barstodraw.Add(ALLBARS,curVolAtPrice);
					
				}
				
				int curTime;

				for (int i= firstBar;i<=lastBar;i++)
				{					
					foreach(KeyValuePair<int, VolAtPrice> timeiterator in this[i])
					{
						curTime=timeiterator.Key;
						
						if (isRTH(curTime)||helper.showAH )
						{
						
							if (!(helper.colorMode==Gom.MPColorMode.TimeColor))						
								curTime=ALLBARS;
								
							else 
							{
								if (!timelist.ContainsKey(curTime))
								{
									timelist.Add(curTime,true);
									timesorted.Add(curTime, true);
									curVolAtPrice=new VolAtPrice();
									barstodraw.Add(curTime,curVolAtPrice);
								}
								
								curVolAtPrice=barstodraw[curTime];
							}
						
							foreach (KeyValuePair<int, VolAndDelta> priceiterator in timeiterator.Value)
								curVolAtPrice.AddVolume(priceiterator.Key,priceiterator.Value.Delta,priceiterator.Value.TotalVolume);
				}	}	}				
				
				foreach (int k in timesorted.Keys)
					barstodraw[k].Draw(helper,isRTH(k)?helper.ColorMapper[k]:helper.neutralColor,x0,lastVolumes);
				
			}
			
			public void DrawBars(int firstBar, int lastBar, int x0, int maxx)
			{
				Dictionary<int,float> lastVolumes;
				Profile prof;

				Draw(firstBar,lastBar,x0,out lastVolumes);
				
				prof=new Profile(helper,lastVolumes);
				
				prof.DrawLevels(x0);
				
				if (helper.showVn && (helper.plotMode != Gom.MPPlotMode.Bar))
					prof.DrawVN(x0,maxx);					
			}			
		}

		
		class GraphicalHelper
		{
			public Graphics graphics;
			public ChartControl chartcontrol;
			public double min,max;
			public double TickSize;
			public Rectangle bounds;
			public int BarSpacing=0;
			public Color[] ColorMapper=new Color[1440]; 
			public bool showAH=true;
			public Color neutralColor=Color.Black;
			public short alpha=128;
			public int minutestep=30;
			public double starthue=0;
			public double endhue=360;
			public Gom.MPColorMode colorMode=Gom.MPColorMode.TimeColor;
			public int rthlengthminutes;
			public bool rtl=false;
			public bool showPoc=true;
			public bool showVa=true;
			public bool showVwap=true;
			public int pocSize=3;
			public bool allPocVolume=true;
			public Color colorPoc=Color.Red;
			public Color colorVa=Color.Blue;			
			public Color colorVwap=Color.Green;
			public Color colorHvn=Color.Green;
			public Color colorLvn=Color.Red;
			public bool showVn=true;
			public int vnSize=5;
			public int accCount=1;
			public int nbSmooth=0;
			public float scale=0.005F; 
			public Gom.MPPlotMode plotMode=Gom.MPPlotMode.Daily;
			public Indicator indic;
			public DateTime startDate=new DateTime();
			
			static private  Color ColorFromHSV(double hue)
			{
				int hi = Convert.ToInt32(Math.Floor(hue / 60)) % 6;
				double f = hue / 60 - Math.Floor(hue / 60);
			
				int v = 255;
				int p = 0;
				int q = Convert.ToInt32(255 * (1 - f));
				int t = Convert.ToInt32(255 * f);
			
				if (hi == 0) return Color.FromArgb(255, v, t, p);
				else if (hi == 1) return Color.FromArgb(255, q, v, p);
				else if (hi == 2) return Color.FromArgb(255, p, v, t);
				else if (hi == 3) return Color.FromArgb(255, p, q, v);
				else if (hi == 4) return Color.FromArgb(255, t, p, v);
				else return Color.FromArgb(255, v, p, q);
			}
			
					
			public void FillRectangle(int priceTick,Color color,float x0, float initialvolume,float volume)
			{
				double priceLower = ((double)priceTick-0.5)*TickSize ; //;- GHelper.TickSize / 2;
				float yLower = GetYPos(priceLower);
				float yUpper = GetYPos(priceLower+ TickSize);
				float height = Math.Max(1, Math.Abs(yUpper - yLower)-BarSpacing );
				
				using (SolidBrush brush=new SolidBrush(Color.FromArgb(alpha,color)))
				{
				if (!rtl)
					graphics.FillRectangle(brush, x0+initialvolume*scale, yUpper, volume*scale, height);
				else
					graphics.FillRectangle(brush, x0-(initialvolume+volume)*scale, yUpper, volume*scale, height);				
                }
			}

				
			public void FillRectangleSpecialBars(int priceTick,Color color,float x0,float volume)
			{
				double priceLower = ((double)priceTick-0.5)*TickSize ; 
				float yLower = GetYPos(priceLower);
				float yUpper = GetYPos(priceLower+ TickSize);
				float height = Math.Max(1, Math.Abs(yUpper - yLower)-BarSpacing );


				if (height<pocSize)
				{
					yUpper=GetYPos(priceTick*TickSize)-(float)Math.Ceiling(pocSize/2.0F);
					height=pocSize;
				}

                using (SolidBrush brush = new SolidBrush(color))
                {
                    if (!rtl)
                        graphics.FillRectangle(brush, x0, yUpper, volume * scale, height);
                    else
                        graphics.FillRectangle(brush, x0 - volume * scale, yUpper, volume * scale, height);
                }
			}

            public void DrawLine(int priceTick, Color color, float x0, float initialvolume,float maxx )
            {
                double price = priceTick * TickSize;
                float y = GetYPos(price);

                using (Pen pen = new Pen(color))
                {
                    if (!rtl)
                        graphics.DrawLine(pen, x0 + initialvolume * scale, y, maxx, y);
					else
						graphics.DrawLine(pen, maxx,y,x0-initialvolume*scale, y);				
                }
            }
			
			 public float GetYPos(double price)
			{
				#if NT7
					return chartcontrol.GetYByValue(indic,price);
				#else			
					return (float) ((bounds.Y + bounds.Height) - 
						((price - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height);
				#endif
			}
					
			
			public void ComputeColorMapper()
			{
				double hue;
				
				int nbclasses=rthlengthminutes/minutestep;
				
				if (nbclasses*minutestep<rthlengthminutes) 
					nbclasses++;
				
				double step=(endhue-starthue)/(nbclasses);
				
				for (int j=0;j<nbclasses;j++)
				{
				 	hue=starthue+j*step;
					
					if (hue>359) hue-=360;
					if (hue<0) hue +=360;				
					
					ColorMapper[j]=ColorFromHSV(hue);
				}
			}		
			

		}
		
		int startbar=-1;
		int lastcalcbar=-1;
		bool firstinitdone=false;
		System.Windows.Forms.KeyEventHandler keyEvtH;

        VolAtTimeBars bartimevols;
        VolAtTimeBar curvolattime;

        List<int> sessions=new List<int>();
			
		TimeSpan startTime=new TimeSpan(15,30,00);
		int starttimeminutes;
		
		TimeSpan rthLength=new TimeSpan(6,45,00);		
		
		GraphicalHelper GHelper=new GraphicalHelper();
		Font myFont=new Font("Arial",8);
		
		private void chart_KeyDown (object sender, KeyEventArgs e)
		{
			
			if (e.KeyCode==Keys.Space)
				GHelper.showAH=!GHelper.showAH;
			
			if (e.KeyCode==Keys.Multiply)
			{	
				if (e.Modifiers==Keys.Control)
					GHelper.rtl=!GHelper.rtl;
				else
					GHelper.showVn = !GHelper.showVn;
			}
				
			if (e.KeyCode==Keys.Divide)
			{
				if (GHelper.plotMode == Gom.MPPlotMode.FromDate)
					GHelper.plotMode=Gom.MPPlotMode.Daily;
				else
					GHelper.plotMode++;
			}
			
			if (e.KeyCode==Keys.Add)
			{	
				if (e.Modifiers==(Keys.Control|Keys.Alt))
					GHelper.accCount++;
	
				else if (e.Modifiers==Keys.Control)
					GHelper.nbSmooth++;
				
				else if (e.Modifiers==Keys.Alt)				 
					GHelper.vnSize=Math.Min(100,GHelper.vnSize+1);
				else
					
					GHelper.scale *= 1.1F;				
			}
			
			if (e.KeyCode==Keys.Subtract)
			{				
				if (e.Modifiers==(Keys.Control|Keys.Alt))
					GHelper.accCount=Math.Max(1,GHelper.accCount-1);
				
				else if (e.Modifiers==Keys.Control)
					GHelper.nbSmooth=Math.Max(0,GHelper.nbSmooth-1);

				else if (e.Modifiers==Keys.Alt)
					GHelper.vnSize=Math.Max(0,GHelper.vnSize-1);
				
				else 
					GHelper.scale *= 0.9F;	
			}
					
			if (e.KeyCode==Keys.Decimal)
			{
				if (GHelper.colorMode>=Gom.MPColorMode.Delta)
					GHelper.colorMode=Gom.MPColorMode.NoColor;
				else	
					GHelper.colorMode++;
			}
		}
		
        protected override void GomInitialize()
        {
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			
			starttimeminutes=(int)startTime.TotalMinutes;
			GHelper.rthlengthminutes=(int)rthLength.TotalMinutes;
			
			GHelper.ComputeColorMapper();
			
			bartimevols=new VolAtTimeBars(GHelper);
			
			sessions.Add(0);
			
			GHelper.startDate=DateTime.Now.Date.Add(startTime);
			
		}
				
		
        protected override void GomOnBarUpdate()
        {
			
			if ((lastcalcbar<CurrentBar)) 
			{
				bartimevols.Add(CurrentBar,new VolAtTimeBar());
				curvolattime=bartimevols[CurrentBar];
				
			}					
			
			if (Bars.SessionBreak)
				sessions.Add(CurrentBar);
				
			if (startbar==-1)
				startbar=CurrentBar;
			
			lastcalcbar=CurrentBar;
        }

		
		protected override void GomOnMarketDataWithTime(DateTime tickTime,TickTypeEnum tickType,double price,int volume,bool firstTickOfBar)	
		{
			
			if (volume==0) return;

			int minute=Convert.ToInt16(Math.Floor((tickTime-Time[0].Date).TotalMinutes)-starttimeminutes);
				
			int time;
				if (minute<0) 
					time=PRE_RTH;
				else if (minute >= GHelper.rthlengthminutes)
					time=POST_RTH;
				else
					time=minute/GHelper.minutestep;

			curvolattime.AddVolume(time,Convert.ToInt32(price/Bars.Instrument.MasterInstrument.TickSize),CalcDelta(tickType,price,volume),volume);
		}

		
		private int GetXPos(int barNum)
		{
			#if NT7 
				return ChartControl.GetXByBarIdx(BarsArray[0],barNum);
			#else
				return ChartControl.GetXByBarIdx(barNum);
			#endif
		}
	
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{

			GHelper.chartcontrol=ChartControl;
			GHelper.graphics=graphics;
			GHelper.max=max;
			GHelper.min=min;
			GHelper.bounds=bounds;

						
			if (Bars==null)	return;

			#if NT7
				int lastBar		= Math.Min(this.LastBarIndexPainted, Bars.Count - 1);
				int firstBar	= this.FirstBarIndexPainted;
			#else
				int lastBar		= Math.Min(ChartControl.LastBarPainted, Bars.Count - 1);
				int firstBar	= Math.Max((lastBar - ChartControl.BarsPainted) + 1,0);
			#endif
								
			
			firstBar=Math.Max(startbar,firstBar);
			lastBar=Math.Min(lastcalcbar,lastBar);
			
			int sideProfile=(GHelper.rtl)?bounds.Right:bounds.Left;
			int sideOpposite=(GHelper.rtl)?bounds.Left:bounds.Right;
			
			if (!firstinitdone)
			{	
				firstinitdone=true;
				keyEvtH=new System.Windows.Forms.KeyEventHandler(this.chart_KeyDown);				
				this.ChartControl.ChartPanel.KeyDown += keyEvtH;
				GHelper.TickSize=Bars.Instrument.MasterInstrument.TickSize;
				GHelper.indic=this;
				
			}			
			
			int x0;

			if (GHelper.plotMode==Gom.MPPlotMode.Daily)
			{			
				int sessnum=-1;
			
				do
				{
					sessnum++;
				}while ((sessions[sessnum]<firstBar)&&(sessnum<(sessions.Count-1)));

				if ((sessnum>0) && ((sessions[sessnum]>lastBar)||GHelper.rtl))
					sessnum--;
				
				do
				{	
					if (sessions[sessnum]<lastBar)
					{
						if (!GHelper.rtl)
							x0=Math.Max(GetXPos(sessions[sessnum]),sideProfile);
						else
						{	
							if ((sessnum==(sessions.Count-1))||(sessions[sessnum+1]>lastBar))
								x0=sideProfile;
							else
								x0=GetXPos(sessions[sessnum+1]);
						}
						
						int fb=sessions[Math.Max(0,sessnum-(GHelper.accCount-1))];
						int lb;
						int maxy;
						
						if (sessnum<(sessions.Count-1))
						{
							lb=Math.Min(sessions[sessnum+1]-1,lastBar);
							maxy=(GHelper.rtl)?GetXPos(sessions[sessnum]):GetXPos(lb);
						}
							
						else
						{
							lb=lastBar;
							maxy=(GHelper.rtl)?GetXPos(sessions[sessnum]):sideOpposite;
						}
						
						bartimevols.DrawBars(fb,lb,x0,maxy);						
					}					
					sessnum++;
				}while (sessnum<sessions.Count) ;
			}
			

			if (GHelper.plotMode==Gom.MPPlotMode.Bar)
			{
				for (int j=firstBar;j<=lastBar;j++)
				{						
					x0=GetXPos(j)+((GHelper.rtl)?-1:1)*3;		
					bartimevols.DrawBars(Math.Max(0,j-(GHelper.accCount-1)),j,x0,sideOpposite);
				}		
			
			}
			
			if (GHelper.plotMode==Gom.MPPlotMode.All)
				bartimevols.DrawBars(startbar,lastBar,sideProfile,sideOpposite);							
			
			
			if (GHelper.plotMode==Gom.MPPlotMode.OnScreen)
				bartimevols.DrawBars(firstBar,lastBar,sideProfile,sideOpposite);	
			
			if (GHelper.plotMode==Gom.MPPlotMode.FromDate)
			{ 
				int fb=Bars.GetBar(GHelper.startDate);
				
				float xpos;
				
				if (fb<lastBar)
					bartimevols.DrawBars(fb,lastBar,(fb<firstBar)?sideProfile:GetXPos(fb),sideOpposite);
			}
			using(SolidBrush brush=new SolidBrush(Color.Black))
			{
				string Msg=	"GomMP - mode:" + GHelper.plotMode + ((GHelper.plotMode==Gom.MPPlotMode.FromDate)?(" "+GHelper.startDate.ToString()):"")+"  - color:"+GHelper.colorMode+" - nb bars/days:"+GHelper.accCount+
							" - smoothing:"+GHelper.nbSmooth+" - min peak size:"+GHelper.vnSize+"% of POC";
				
				graphics.DrawString(Msg,myFont,brush,bounds.Left+130,bounds.Bottom-13);
			}
		}
		
	
		public sealed override void Dispose()
        {
			if (keyEvtH != null)
			{	
				this.ChartControl.ChartPanel.KeyDown -= keyEvtH;
			   	keyEvtH=null;
			}
				base.Dispose();

			bartimevols=null;
			curvolattime=null;
			
			myFont.Dispose();
			//GC.Collect();
			//GC.WaitForPendingFinalizers();
		}
		
        #region Properties

			
		[Description("Plotting mode.")]
		[Category("0. Main Settings")]
		[Gui.Design.DisplayNameAttribute("0. Plot Mode")]
        public Gom.MPPlotMode PlotMode
        {
            get { return GHelper.plotMode; }
            set { GHelper.plotMode = value; }
        }
		
		[Description("Date Start. Only used if PlotMode=FromDate")]
		[Category("0. Main Settings")]
		[Gui.Design.DisplayNameAttribute("Obis. Start Date")]
        public DateTime StartDate
        {
            get { return GHelper.startDate; }
            set { GHelper.startDate = value; }
        }	
		
		[Description("Color Mode. Time Based color change, Delta and NoColor. Can be toggled using . Key")]
		[Category("0. Main Settings")]
		[Gui.Design.DisplayNameAttribute(". Color Bar Mode")]
        public Gom.MPColorMode ColorMode
        {
            get { return GHelper.colorMode; }
            set { GHelper.colorMode = value; }
        }	
		
		[Description("Days/Bars to accumulate. Can be incremented/decremented using Ctrl Alt + and Ctrl Alt - keys")]
		[Category("0. Main Settings")]
		[Gui.Design.DisplayNameAttribute("2. Days/Bars to accumulate")]
        public int AccCount
        {
            get { return GHelper.accCount; }
            set { if (value>0) GHelper.accCount = value; }
        }
		
		[Description("Number of smoothing bars. Can be toggled using Ctrl + and Ctrl -")]
		[Category("0. Main Settings")]
		[Gui.Design.DisplayNameAttribute("3. Nb smoothing bars")]
        public int NbSmooth
        {
            get { return GHelper.nbSmooth; }
            set { if (value>=0) GHelper.nbSmooth = value; }
        }
		
		[Description("RTH Start Time")]
        [Category("1. Timing Settings")]
		[Gui.Design.DisplayNameAttribute("1. RTH Start Time")]
		[XmlIgnore()]
        public TimeSpan StartTime
        {
            get { return startTime; }
            set { startTime = value;
				  starttimeminutes=(int)startTime.TotalMinutes;
				}
        }
		
		[Browsable(false)]
		public string StartTimeSerialize
			{
    			get { return startTime.ToString(); }
     			set { startTime = TimeSpan.Parse(value); }
			}
			
		[Description("RTH Session Length")]
        [Category("1. Timing Settings")]
		[Gui.Design.DisplayNameAttribute("2. RTH Session Length")]
		[XmlIgnore()]
        public TimeSpan RTHLength
        {
            get { return rthLength; }
            set { 
					rthLength = value; 
					GHelper.rthlengthminutes=(int)rthLength.TotalMinutes;
					GHelper.ComputeColorMapper();
				}
        }
		
		[Browsable(false)]
		public string RTHLengthSerialize
			{
    			get { return rthLength.ToString(); }
     			set { rthLength = TimeSpan.Parse(value); }
			}
			
		[Description("Show AH trades. Can be toggled using space bar")]
        [Category("1. Timing Settings")]
		[Gui.Design.DisplayNameAttribute("3. Show AH Trades")]
        public bool ShowAH
        {
            get { return GHelper.showAH; }
            set { GHelper.showAH = value; }
        }	
			
		[XmlIgnore()]
		[Description("Neutral Color. Used as a clor for AH and if fancy colors is disabled")]
		[Category("2. Display Settings")]
		[Gui.Design.DisplayName("1. Neutral Color")]
		
		public Color NeutralColor
		{
			get { return GHelper.neutralColor; }
			set {  GHelper.neutralColor=value;}
		}
		
		[Browsable(false)]
		public string NeutralColorSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(GHelper.neutralColor); }
     			set { GHelper.neutralColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}

		[Description("Alpha Channel for the color. 0-255 (0= full transparent, 255= full opacity")]
		[Category("2. Display Settings")]
		[Gui.Design.DisplayNameAttribute("2. Alpha")]
        public short Alpha
        {
            get { return GHelper.alpha; }
            set { 
					GHelper.alpha = value;
					GHelper.ComputeColorMapper();
				}
        }	
		
		[Description("Spacing between the volume bars.")]
		[Category("2. Display Settings")]
		[Gui.Design.DisplayNameAttribute("3. Bar spacing")]
		public int BarSpacing
		{
			get { return GHelper.BarSpacing; }
			set 
			{ 
				GHelper.BarSpacing = Math.Min(5, Math.Max(0,  value));
			}
		}
		
			
		[Description("RTL Mode : If true Right to Left, if false Left To Right. Can be toggles using Ctrl * key")]
		[Category("2. Display Settings")]
		[Gui.Design.DisplayNameAttribute("4. Right To Left")]
        public bool RTL
        {
            get { return GHelper.rtl; }
            set { GHelper.rtl = value; }
        }
		
		[Description("Plotting Scale. Determines lengths of bars. Can be modified using + and - keys")]
		[Category("2. Display Settings")]
		[Gui.Design.DisplayNameAttribute("5. Plotting Scale")]
        public float Scale
        {
            get { return GHelper.scale; }
            set { GHelper.scale = value; }
        }		
		
		[Description("Show POC")]
		[Category("3. POC/VA/VWAP")]
		[Gui.Design.DisplayNameAttribute("1. Show POC")]
        public bool ShowsPOC
        {
            get { return GHelper.showPoc; }
            set { GHelper.showPoc = value; }
        }
		
		[Description("Show VA")]
		[Category("3. POC/VA/VWAP")]
		[Gui.Design.DisplayNameAttribute("2. Show VA")]
        public bool ShowsVA
        {
            get { return GHelper.showVa; }
            set { GHelper.showVa = value; }
        }
		
		[Description("Show VWAP")]
		[Category("3. POC/VA/VWAP")]
		[Gui.Design.DisplayNameAttribute("3. Show VWAP")]
        public bool ShowsVWAP
        {
            get { return GHelper.showVwap; }
            set { GHelper.showVwap = value; }
        }

		[Description("Minimum size of POC bar and others (vwap etc)")]
		[Category("3. POC/VA/VWAP")]
		[Gui.Design.DisplayNameAttribute("4. POC etc bar min height")]
        public int PocSize
        {
            get { return GHelper.pocSize; }
            set { 
					if (value>0)
					GHelper.pocSize = value; 
			}
        }
		
		[Description("Make all special bars POC size")]
		[Category("3. POC/VA/VWAP")]
		[Gui.Design.DisplayNameAttribute("5. All bars POC size")]
        public bool AllPocVolume
        {
            get { return GHelper.allPocVolume; }
            set { GHelper.allPocVolume = value; }
        }
		
				
		[Description("POC Color")]        
		[Category("3. POC/VA/VWAP")]
		[Gui.Design.DisplayNameAttribute("6. POC Color")]
		[XmlIgnore()]	
		public Color ColorPOC
		{
			get { return GHelper.colorPoc; }
			set {  GHelper.colorPoc=value; }
		}
			
		[Browsable(false)]
		public string ColorPOCSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(GHelper.colorPoc); }
     			set { GHelper.colorPoc = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}

		[Description("VA Color")]        
		[Category("3. POC/VA/VWAP")]
		[Gui.Design.DisplayNameAttribute("7. VA Color")]
		[XmlIgnore()]	
		public Color ColorVA
		{
			get { return GHelper.colorVa; }
			set {  GHelper.colorVa=value; }
		}
			
		[Browsable(false)]
		public string ColorVASerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(GHelper.colorVa); }
     			set { GHelper.colorVa = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}
			
		[Description("VWAP Color")]        
		[Category("3. POC/VA/VWAP")]
		[Gui.Design.DisplayNameAttribute("8. VWAP Color")]
		[XmlIgnore()]	
		public Color ColorVWAP
		{
			get { return GHelper.colorVwap; }
			set {  GHelper.colorVwap=value; }
		}
			
		[Browsable(false)]
		public string ColorVWAPSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(GHelper.colorVwap); }
     			set { GHelper.colorVwap = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}
			
		[Description("Show HVN/LVN. Can be toggled using * key")]
		[Category("4. HVN/LVN")]
		[Gui.Design.DisplayNameAttribute("1. Show HVN/LVN")]
        public bool ShowVN
        {
            get { return GHelper.showVn; }
            set { GHelper.showVn = value; }
        }			
			
		[Description("Minimum size of peaks, expressed as percentage of the POC. Can be incremented decremented using Alt + and Alt -")]
		[Category("4. HVN/LVN")]
		[Gui.Design.DisplayNameAttribute("2. HVN/LVN Min Size")]
        public int VNSize
        {
            get { return GHelper.vnSize; }
            set { GHelper.vnSize = value; }
        }	
			
		[Description("HVN Color")]        
		[Category("4. HVN/LVN")]
		[Gui.Design.DisplayNameAttribute("3. HVN Color")]
		[XmlIgnore()]	
		public Color ColorHVN
		{
			get { return GHelper.colorHvn; }
			set {  GHelper.colorHvn=value; }
		}
			
		[Browsable(false)]
		public string ColorHVNSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(GHelper.colorHvn); }
     			set { GHelper.colorHvn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}
			
		[Description("LVN Color")]        
		[Category("4. HVN/LVN")]
		[Gui.Design.DisplayNameAttribute("4. LVN Color")]
		[XmlIgnore()]	
		public Color ColorLVN
		{
			get { return GHelper.colorLvn; }
			set {  GHelper.colorLvn=value; }
		}
			
		[Browsable(false)]
		public string ColorLVNSerialize
			{
    			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(GHelper.colorLvn); }
     			set { GHelper.colorLvn = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
			}
		
			
			
		[Description("Color Resolution in minutes (must be >=1). If set to 15, there is one different color every 15 minutes. Lowering the setting will increase color smoothness but will increase memory footprint ")]
		[Category("5. Time Color Settings")]
		[Gui.Design.DisplayNameAttribute("1. Color Resolution")]
        public int MinuteStep
        {
            get { return GHelper.minutestep; }
            set {  
					GHelper.minutestep = value; 
				    GHelper.ComputeColorMapper();
				}
        }
		
		[Description("Starting Color Hue. Must be between 0-360 (see Wikipedia on color Hue")]
		[Category("5. Time Color Settings")]
		[Gui.Design.DisplayNameAttribute("2. Starting Hue")]
        public double StartHue
        {
            get { return GHelper.starthue; }
            set {  
					GHelper.starthue = value; 
					GHelper.ComputeColorMapper();
       			 }
		}
		
		[Description("Ending Color Hue. Must be between 0-360 (see Wikipedia on color Hue")]        
		[Category("5. Time Color Settings")]
		[Gui.Design.DisplayNameAttribute("3. Ending Hue")]
        public double EndHue
        {
            get { return GHelper.endhue; }
            set { 
					GHelper.endhue= value; 
					GHelper.ComputeColorMapper();
				}
        }

		#endregion
    }
	
}

namespace Gom
{
    public enum MPColorMode
    {
        NoColor,
        TimeColor,
        Delta
    }
	
	public enum MPPlotMode
	{
		Daily,
		Bar,
		OnScreen,
		All,
		FromDate
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private GomMP[] cacheGomMP = null;

        private static GomMP checkGomMP = new GomMP();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public GomMP GomMP()
        {
            return GomMP(Input);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public GomMP GomMP(Data.IDataSeries input)
        {
            if (cacheGomMP != null)
                for (int idx = 0; idx < cacheGomMP.Length; idx++)
                    if (cacheGomMP[idx].EqualsInput(input))
                        return cacheGomMP[idx];

            lock (checkGomMP)
            {
                if (cacheGomMP != null)
                    for (int idx = 0; idx < cacheGomMP.Length; idx++)
                        if (cacheGomMP[idx].EqualsInput(input))
                            return cacheGomMP[idx];

                GomMP indicator = new GomMP();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                Indicators.Add(indicator);
                indicator.SetUp();

                GomMP[] tmp = new GomMP[cacheGomMP == null ? 1 : cacheGomMP.Length + 1];
                if (cacheGomMP != null)
                    cacheGomMP.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheGomMP = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.GomMP GomMP()
        {
            return _indicator.GomMP(Input);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.GomMP GomMP(Data.IDataSeries input)
        {
            return _indicator.GomMP(input);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.GomMP GomMP()
        {
            return _indicator.GomMP(Input);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.GomMP GomMP(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.GomMP(input);
        }
    }
}
#endregion
